import 'dart:async';

import 'package:hotech_flutter_utilities/LoginUser.dart';
import 'package:hotech_flutter_utilities/model/login/LoginAuth.dart';
import 'package:hotech_flutter_utilities/model/login/Loginfo.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:hotech_flutter_utilities/Constants.dart';

class HFSharedPreferences {
  static HFSharedPreferences instance;

  static SharedPreferences preferences;
  static FlutterSecureStorage secureStorage;

  static Future<HFSharedPreferences> getInstance() async {
    preferences = await SharedPreferences.getInstance();
    secureStorage = new FlutterSecureStorage();
    return instance != null ? instance : HFSharedPreferences();
  }

  String getString(String key) {
    return preferences.getString(key);
  }

  String getStringDefault(String key, String defaultValue) {
    return preferences.getString(key) ?? defaultValue;
  }

  bool getBool(String key, bool defaultValue) {
    return preferences.getBool(key) ?? defaultValue;
  }

  int getInt(String key, int defaultValue) {
    return preferences.getInt(key) ?? defaultValue;
  }

  double getDouble(String key, double defaultValue) {
    return preferences.getDouble(key) ?? defaultValue;
  }

  Future<HFSharedPreferences> setString(String key, String value) async {
    await preferences.setString(key, value);
    return instance;
  }

  Future<String> getSecureString(String key, String defaultValue) async {
    String secureString = await secureStorage.read(key: key);
    return secureString;
  }

  Future<int> getSecureInt(String key, int defaultValue) async {
    String secureString = await secureStorage.read(key: key);
    int secureInt = int.parse(secureString);
    return secureInt;
  }

  Future<HFSharedPreferences> setBool(String key, bool value) async {
    await preferences.setBool(key, value);
    return instance;
  }

  Future<HFSharedPreferences> setInt(String key, int value) async {
    await preferences.setInt(key, value);
    return instance;
  }

  Future<HFSharedPreferences> setDouble(String key, double value) async {
    await preferences.setDouble(key, value);
    return instance;
  }

  Future<HFSharedPreferences> setSecureString(String key, String value) async {
    await secureStorage.write(key: key, value: value);
    return instance;
  }

  //Change this to delete only the items defined in the method below if you plan on encrypting more than login data!!!
  Future<HFSharedPreferences> invalidateTokens() async {
    await secureStorage.deleteAll();
    return instance;
  }

  Future saveLoginAuth(LoginAuth aLoginAuth) async {
    await setSecureString(DBConst.FDN_ACCESSTOKEN, aLoginAuth.accessToken);
    await setSecureString(DBConst.FDN_TOKENTYPE, aLoginAuth.tokenType);
    await setSecureString(DBConst.FDN_REFRESH_TOKEN, aLoginAuth.refreshToken);
    await setSecureString(DBConst.FDN_EXPIRES_IN, aLoginAuth.expiresIn.toString());
    await setSecureString(DBConst.FDN_SCOPE, aLoginAuth.scope);
  }

  Future<HFSharedPreferences> saveLoginfo(Loginfo loginfo) async {
    await setBool(DBConst.FDN_ACCOUNTNONEXPIRED, loginfo.accountNonExpired);
    await setBool(DBConst.FDN_ACCOUNTNONLOCKED, loginfo.accountNonLocked);
    await setInt(DBConst.FDN_AUTHLEVEL, loginfo.authlevel);
    await setInt(DBConst.FDN_AUTHLEVELFR, loginfo.authlevelfr);
    await setInt(DBConst.FDN_AUTHLEVELSO, loginfo.authlevelso);
    await setInt(DBConst.FDN_AUTHLEVELSR, loginfo.authlevelsr);
    await setString(DBConst.FDN_CODE, loginfo.code);
    await setBool(DBConst.FDN_CREDENTIALSNONEXPIRED, loginfo.credentialsNonExpired);
    await setInt(DBConst.FDN_CURRENTHOTELREFNO, loginfo.currentHotelrefno);
    await setInt(DBConst.FDN_CURRENTID, loginfo.currentid);
    await setInt(DBConst.FDN_DEFAULTHOTELREFNO, loginfo.defaultHotelrefno);
    await setString(DBConst.FDN_DESCRIPTION, loginfo.description);
    await setString(DBConst.FDN_EMAIL, loginfo.email);
    await setString(DBConst.FDN_EMPDEPARTCODE, loginfo.empdepartcode);
    await setInt(DBConst.FDN_EMPDEPARTID, loginfo.empdepartid);
    await setString(DBConst.FDN_EMPPOSCODE, loginfo.empposcode);
    await setInt(DBConst.FDN_EMPPOSID, loginfo.empposid);
    await setBool(DBConst.FDN_ENABLED, loginfo.enabled);
    await setString(DBConst.FDN_FIRSTNAME, loginfo.firstname);
    await setString(DBConst.FDN_GID, loginfo.gid);
    await setBool(DBConst.FDN_HASMULTIHOTEL, loginfo.hasmultihotel);
    await setString(DBConst.FDN_HOTELGIDSTR, loginfo.hotelgidstr);
    await setInt(DBConst.FDN_HOTELMID, loginfo.hotelmid);
    await setString(DBConst.FDN_HOTELNAME, loginfo.hotelname);
    await setInt(DBConst.FDN_HOTELREFNO, loginfo.hotelrefno);
    await setDouble(DBConst.FDN_HOTELNG, loginfo.hotelng);
    await setDouble(DBConst.FDN_HOTELAT, loginfo.hotelat);
    await setString(DBConst.FDN_MOBILETEL, loginfo.mobiletel);
    await setInt(DBConst.FDN_ID, loginfo.id);
    await setString(DBConst.FDN_INSDATETIME, loginfo.insdatetime);
    await setInt(DBConst.FDN_INSUSERID, loginfo.insuserid);
    await setBool(DBConst.FDN_ISACTIVE, loginfo.isactive);
    await setBool(DBConst.FDN_ISBANQUET, loginfo.isbanquet);
    await setBool(DBConst.FDN_ISDEF, loginfo.isdef);
    await setBool(DBConst.FDN_ISEMPSELF, loginfo.isempself);
    await setBool(DBConst.FDN_ISFINREQEMP, loginfo.isfinreqemp);
    await setBool(DBConst.FDN_ISHEAD, loginfo.ishead);
    await setBool(DBConst.FDN_ISHK, loginfo.ishk);
    await setBool(DBConst.FDN_ISLOCRES, loginfo.islocres);
    await setBool(DBConst.FDN_ISPOSONLY, loginfo.isposonly);
    await setBool(DBConst.FDN_ISSPAINFOEMP, loginfo.isspainfoemp);
    await setBool(DBConst.FDN_ISSPARES, loginfo.isspares);
    await setBool(DBConst.FDN_ISSPASALEEMP, loginfo.isspasaleemp);
    await setBool(DBConst.FDN_ISSUPERHOTEL, loginfo.issuperhotel);
    await setBool(DBConst.FDN_ISSUPERUSER, loginfo.issuperuser);
    await setBool(DBConst.FDN_ISTS, loginfo.ists);
    await setBool(DBConst.FDN_ISWAITER, loginfo.iswaiter);
    await setString(DBConst.FDN_LANGCODE, loginfo.langcode);
    await setInt(DBConst.FDN_LANGID, loginfo.langid);
    await setString(DBConst.FDN_LANGSHORT, loginfo.langshort);
    await setString(DBConst.FDN_LASTNAME, loginfo.lastname);
    await setString(DBConst.FDN_LOGINTYPE, loginfo.logintype);
    await setInt(DBConst.FDN_MID, loginfo.mid);
    await setString(DBConst.FDN_PASS, loginfo.pass);
    await setString(DBConst.FDN_REFCODE, loginfo.refcode);
    await setInt(DBConst.FDN_REFID, loginfo.refid);
    await setBool(DBConst.FDN_SELFISH, loginfo.selfish);
    await setString(DBConst.FDN_SHORTCODE, loginfo.shortcode);
    await setString(DBConst.FDN_UPDATETIME, loginfo.updatetime);
    await setInt(DBConst.FDN_UPDTL, loginfo.updtl);
    await setString(DBConst.FDN_USERGROUPIDSTR, loginfo.usergroupidstr);
    await setString(DBConst.FDN_USERGROUPS, loginfo.usergroups);
    await setInt(DBConst.FDN_USERID, loginfo.userid);
    await setInt(DBConst.FDN_VER, loginfo.ver);
    await setString(DBConst.FDN_XCODE, loginfo.xcode);
    return instance;
  }

  Future<String> getAccessToken() {
    return getSecureString(DBConst.FDN_ACCESSTOKEN, LoginUser.loginAuth?.accessToken);
  }

  Future<String> getTokenType() {
    return getSecureString(DBConst.FDN_TOKENTYPE, LoginUser.loginAuth?.refreshToken);
  }

  Future<String> getRefreshToken() {
    return getSecureString(DBConst.FDN_REFRESH_TOKEN, LoginUser.loginAuth?.tokenType);
  }

  Future<int> getExpiresIn() {
    return getSecureInt(DBConst.FDN_EXPIRES_IN, LoginUser.loginAuth.expiresIn);
  }

  int getUserID() {
    return getInt(DBConst.FDN_ID, LoginUser.logInfo?.id);
  }

  String getUserCode() {
    return getStringDefault(DBConst.FDN_USERCODE, LoginUser.logInfo?.code);
  }

  int getHotelRefNo() {
    return getInt(DBConst.FDN_HOTELREFNO, LoginUser.logInfo?.hotelrefno);
  }

  String getHotelName() {
    return getStringDefault(DBConst.FDN_HOTELNAME, LoginUser.logInfo?.hotelname);
  }

  String getShortCode() {
    return getStringDefault(DBConst.FDN_SHORTCODE, LoginUser.logInfo?.shortcode);
  }

  int getAuthLevel() {
    return getInt(DBConst.FDN_AUTHLEVEL, LoginUser.logInfo?.authlevel);
  }

  String getEmail() {
    return getStringDefault(DBConst.FDN_EMAIL, LoginUser.logInfo?.email);
  }

  String getUserGid() {
    return getStringDefault(DBConst.FDN_USERGID, LoginUser.logInfo?.gid);
  }

  bool getIsSelish() {
    return getBool(DBConst.FDN_SELFISH, LoginUser.logInfo?.selfish);
  }

  int getHotelMid() {
    return getInt(DBConst.FDN_HOTELMID, LoginUser.logInfo?.hotelmid);
  }

  List<int> getEmpHotelIDs() {
    return getStringDefault(DBConst.FDN_EMPHOTELIDS, '')
        .split(
          ',',
        )
        .map((str) => int.parse(str))
        .toList();
  }

  String getDepartNo() {
    return getStringDefault(DBConst.FDN_DEPARTNO, null);
  }

  bool getIsPosOnly() {
    return getBool(DBConst.FDN_ISPOSONLY, LoginUser.logInfo?.isposonly);
  }

  bool getIsHk() {
    return getBool(DBConst.FDN_ISHK, LoginUser.logInfo?.ishk);
  }

  bool getIshead() {
    return getBool(DBConst.FDN_ISHEAD, LoginUser.logInfo?.ishead);
  }

  bool getIsSuperhotel() {
    return getBool(DBConst.FDN_ISSUPERHOTEL, LoginUser.logInfo?.issuperhotel);
  }

  int getUserMid() {
    return getInt(DBConst.FDN_USERMID, LoginUser.logInfo?.mid);
  }

  String getHotelgidstr() {
    return getStringDefault(DBConst.FDN_HOTELGIDSTR, '');
  }

  double getHotelLat() {
    return getDouble(DBConst.FDN_HOTELAT, LoginUser.logInfo?.hotelat);
  }

  double getHotelLng() {
    return getDouble(DBConst.FDN_HOTELNG, LoginUser.logInfo?.hotelng);
  }
}
