import 'package:hotech_flutter_utilities/Constants.dart';

class Faq {
  List<FaqCategory> categories;
  List<FaqElement> faqs;
  Map<String, dynamic> jsonMap;

  Faq({
    this.categories,
    this.faqs,
    this.jsonMap,
  });

  factory Faq.fromJson(Map<String, dynamic> json) => new Faq(
        categories: new List<FaqCategory>.from(json[DBConst.FDN_CATEGORIES].map((x) => FaqCategory.fromJson(x))),
        faqs: new List<FaqElement>.from(json[DBConst.FDN_FAQS].map((x) => FaqElement.fromJson(x))),
        jsonMap: json,
      );

  Map<String, dynamic> toJson() => {
        DBConst.FDN_CATEGORIES: new List<dynamic>.from(categories.map((x) => x.toJson())),
        DBConst.FDN_FAQS: new List<dynamic>.from(faqs.map((x) => x.toJson())),
      };

  @override
  String toString() {
    return 'Faq{categories: $categories, faqs: $faqs, jsonMap: $jsonMap}';
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is Faq && categories == other.categories && faqs == other.faqs && jsonMap == other.jsonMap;

  @override
  int get hashCode => categories.hashCode ^ faqs.hashCode ^ jsonMap.hashCode;
}

class FaqCategory {
  int id;
  String version;
  Map<String, dynamic> jsonMap;

  FaqCategory({
    this.id,
    this.version,
    this.jsonMap,
  });

  factory FaqCategory.fromJson(Map<String, dynamic> json) => new FaqCategory(
        id: json[DBConst.FDN_ID],
        version: json[DBConst.FDN_VERSION],
        jsonMap: json,
      );

  Map<String, dynamic> toJson() => {
        DBConst.FDN_ID: id,
        DBConst.FDN_VERSION: version,
      };

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is FaqCategory && id == other.id && version == other.version && jsonMap == other.jsonMap;

  @override
  int get hashCode => id.hashCode ^ version.hashCode ^ jsonMap.hashCode;

  @override
  String toString() {
    return 'Category{id: $id, version: $version, jsonMap: $jsonMap}';
  }
}

class FaqElement {
  String question;
  String answer;
  int category;
  Map<String, dynamic> jsonMap;

  FaqElement({
    this.question,
    this.answer,
    this.category,
    this.jsonMap,
  });

  factory FaqElement.fromJson(Map<String, dynamic> json) => new FaqElement(
        question: json[DBConst.FDN_QUESTION],
        answer: json[DBConst.FDN_ANSWER],
        category: json[DBConst.FDN_CATEGORY],
        jsonMap: json,
      );

  @override
  String toString() {
    return 'FaqElement{question: $question, answer: $answer, category: $category, jsonMap: $jsonMap}';
  }

  Map<String, dynamic> toJson() => {
        DBConst.FDN_QUESTION: question,
        DBConst.FDN_ANSWER: answer,
        DBConst.FDN_CATEGORY: category,
      };

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is FaqElement &&
          question == other.question &&
          answer == other.answer &&
          category == other.category &&
          jsonMap == other.jsonMap;

  @override
  int get hashCode => question.hashCode ^ answer.hashCode ^ category.hashCode ^ jsonMap.hashCode;
}
