import 'package:hotech_flutter_utilities/Constants.dart';

///weather response, the immediate result received after calling the weather service
class ForecastRes {
  List<ForecastItem> list;
  String cod;
  int cnt;
  City city;

  ForecastRes({
    this.list,
    this.cod,
    this.cnt,
    this.city,
  });

  factory ForecastRes.fromJson(Map<String, dynamic> json) => new ForecastRes(
        list: new List<ForecastItem>.from(json[Consts.PARAM_LIST].map((x) => ForecastItem.fromJson(x))),
        cod: json[Consts.PARAM_COD],
        cnt: json[Consts.PARAM_CNT],
        city: City.fromJson(json[Consts.PARAM_CITY]),
      );

  Map<String, dynamic> toJson() => {
        Consts.PARAM_LIST: new List<dynamic>.from(list.map((x) => x.toJson())),
        Consts.PARAM_COD: cod,
        Consts.PARAM_CNT: cnt,
        Consts.PARAM_CITY: city.toJson(),
      };
}

class WeatherNowRes {
  List<Weather> weather;
  String base;
  Main main;
  int dt;
  int id;
  String name;
  int cod;

  WeatherNowRes({
    this.weather,
    this.base,
    this.main,
    this.dt,
    this.id,
    this.name,
    this.cod,
  });

  factory WeatherNowRes.fromJson(Map<String, dynamic> json) => new WeatherNowRes(
        weather: new List<Weather>.from(json[Consts.PARAM_WEATHER].map((x) => Weather.fromJson(x))),
        base: json[Consts.PARAM_BASE],
        main: Main.fromJson(json[Consts.PARAM_MAIN]),
        dt: json[Consts.PARAM_DT],
        id: json[Consts.PARAM_ID],
        name: json[Consts.PARAM_NAME],
        cod: json[Consts.PARAM_COD],
      );

  Map<String, dynamic> toJson() => {
        Consts.PARAM_WEATHER: new List<dynamic>.from(weather.map((x) => x.toJson())),
        Consts.PARAM_BASE: base,
        Consts.PARAM_MAIN: main.toJson(),
        Consts.PARAM_DT: dt,
        Consts.PARAM_ID: id,
        Consts.PARAM_NAME: name,
        Consts.PARAM_COD: cod,
      };
}

///represents 1 day of forecast

class ForecastItem {
  int dt;
  int sunrise;
  int sunset;
  //double pressure;
  int humidity;
  Weather weather;
  int clouds;
  Temp temp;

  ForecastItem({
    this.dt,
    this.weather,
    this.clouds,
    this.sunrise,
    this.sunset,
    //this.pressure,
    this.humidity,
    this.temp,
  });

  factory ForecastItem.fromJson(Map<String, dynamic> json) => new ForecastItem(
        dt: json[Consts.PARAM_DT],
        weather: Weather.fromJson(json[Consts.PARAM_WEATHER][0]),
        clouds: json[Consts.PARAM_CLOUDS],
        sunrise: json[Consts.PARAM_SUNRISE],
        sunset: json[Consts.PARAM_SUNSET],
        humidity: json[Consts.PARAM_HUMIDITY],
        temp: Temp.fromJson(json[Consts.PARAM_TEMP]),
      );

  Map<String, dynamic> toJson() => {
        Consts.PARAM_DT: dt,
        Consts.PARAM_WEATHER: weather,
        Consts.PARAM_CLOUDS: clouds,
        Consts.PARAM_SUNRISE: sunrise,
        Consts.PARAM_SUNSET: sunset,
        Consts.PARAM_HUMIDITY: humidity,
        Consts.PARAM_TEMP: temp,
      };
}

class Temp {
  double day;
  double min;
  double max;
  double eve;
  double morn;

  Temp({
    this.day,
    this.min,
    this.max,
    this.eve,
    this.morn,
  });

  factory Temp.fromJson(Map<String, dynamic> json) => new Temp(
        day: json[Consts.PARAM_DAY].toDouble(),
        min: json[Consts.PARAM_MIN].toDouble(),
        max: json[Consts.PARAM_MAX].toDouble(),
        eve: json[Consts.PARAM_EVE].toDouble(),
        morn: json[Consts.PARAM_MORN].toDouble(),
      );

  Map<String, dynamic> toJson() => {
        Consts.PARAM_DAY: day,
        Consts.PARAM_MIN: min,
        Consts.PARAM_MAX: max,
        Consts.PARAM_EVE: eve,
        Consts.PARAM_MORN: morn,
      };
}

///data about the city
class City {
  int id;
  String name;
  String country;
  int population;
  int timezone;

  City({
    this.id,
    this.name,
    this.country,
    this.population,
    this.timezone,
  });

  factory City.fromJson(Map<String, dynamic> json) => new City(
        id: json[Consts.PARAM_ID],
        name: json[Consts.PARAM_NAME],
        country: json[Consts.PARAM_COUNTRY],
        population: json[Consts.PARAM_POPULATION],
        timezone: json[Consts.PARAM_TIMEZONE],
      );

  Map<String, dynamic> toJson() => {
        Consts.PARAM_ID: id,
        Consts.PARAM_NAME: name,
        Consts.PARAM_COUNTRY: country,
        Consts.PARAM_POPULATION: population,
        Consts.PARAM_TIMEZONE: timezone,
      };
}

class Main {
  double temp;
  double pressure;
  int humidity;
  double tempMin;
  double tempMax;

  Main({
    this.temp,
    this.pressure,
    this.humidity,
    this.tempMin,
    this.tempMax,
  });

  factory Main.fromJson(Map<String, dynamic> json) => new Main(
        temp: json[Consts.PARAM_TEMP].toDouble(),
        pressure: json[Consts.PARAM_PRESSURE].toDouble(),
        humidity: json[Consts.PARAM_HUMIDITY],
        tempMin: json[Consts.PARAM_TEMP_MIN].toDouble(),
        tempMax: json[Consts.PARAM_TEMP_MAX].toDouble(),
      );

  Map<String, dynamic> toJson() => {
        Consts.PARAM_TEMP: temp,
        Consts.PARAM_PRESSURE: pressure,
        Consts.PARAM_HUMIDITY: humidity,
        Consts.PARAM_TEMP_MIN: tempMin,
        Consts.PARAM_TEMP_MAX: tempMax,
      };
}

class Weather {
  int id;
  String main;
  String description;
  String icon;

  Weather({
    this.id,
    this.main,
    this.description,
    this.icon,
  });

  factory Weather.fromJson(Map<String, dynamic> json) => new Weather(
        id: json[Consts.PARAM_ID],
        main: json[Consts.PARAM_MAIN],
        description: json[Consts.PARAM_DESCRIPTION],
        icon: json[Consts.PARAM_ICON],
      );

  Map<String, dynamic> toJson() => {
        Consts.PARAM_ID: id,
        Consts.PARAM_MAIN: main,
        Consts.PARAM_DESCRIPTION: description,
        Consts.PARAM_ICON: icon,
      };
}

class WeatherData {
  final DateTime date;
  final String name;
  final double tempDay;
  final double tempMin;
  final double tempMax;
  final String main;
  final int statusCode;
  final String icon;

  ForecastRes weatherRes;

  WeatherData._(
      {this.date, this.name, this.tempDay, this.tempMin, this.tempMax, this.main, this.statusCode, this.icon});

  ///Creates an easy to use weather data package from a [ForecastRes] and [day] representing the days from today the
  ///forecast is in, 0 being today
  factory WeatherData(ForecastRes weatherRes, {int day = 0}) {
    return WeatherData._(
      date: new DateTime.fromMillisecondsSinceEpoch(weatherRes.list[day].dt * 1000, isUtc: false),
      name: weatherRes.city.name,
      tempDay: weatherRes.list[day].temp.day.toDouble(),
      tempMin: weatherRes.list[day].temp.min.toDouble(),
      tempMax: weatherRes.list[day].temp.max.toDouble(),
      main: weatherRes.list[day].weather.main,
      statusCode: weatherRes.list[day].weather.id,
      icon: weatherRes.list[day].weather.icon,
    );
  }
}

class WeatherNowData {
  final DateTime date;
  final String name;
  final double temp;
  final String main;
  final String icon;

  WeatherNowRes weatherRes;

  WeatherNowData._({this.date, this.name, this.temp, this.main, this.icon});

  factory WeatherNowData(WeatherNowRes weatherRes) {
    return WeatherNowData._(
      date: new DateTime.fromMillisecondsSinceEpoch(weatherRes.dt * 1000, isUtc: false),
      name: weatherRes.name,
      temp: weatherRes.main.temp.toDouble(),
      main: weatherRes.weather[0].main,
      icon: weatherRes.weather[0].icon,
    );
  }

  factory WeatherNowData.fromJson(Map<String, dynamic> json) {
    return WeatherNowData._(
      date: new DateTime.fromMillisecondsSinceEpoch(json[Consts.PARAM_DT] * 1000, isUtc: false),
      name: json[Consts.PARAM_NAME],
      temp: json[Consts.PARAM_MAIN][Consts.PARAM_TEMP].toDouble(),
      main: json[Consts.PARAM_WEATHER][0][Consts.PARAM_MAIN],
      icon: json[Consts.PARAM_WEATHER][0][Consts.PARAM_ICON],
    );
  }
}
