import 'dart:async';
import 'dart:convert';

import 'package:hotech_flutter_utilities/LoginUser.dart';
import 'package:hotech_flutter_utilities/Constants.dart';
import 'package:hotech_flutter_utilities/model/SingleDataResult.dart';
import 'package:hotech_flutter_utilities/model/login/LoginAuth.dart';
import 'package:hotech_flutter_utilities/model/login/Loginfo.dart';
import 'package:hotech_flutter_utilities/service/LoginHelper.dart';
import 'package:hotech_flutter_utilities/service/ServiceResponse.dart';
import 'package:hotech_flutter_utilities/service/ServiceFactory.dart';
import 'package:hotech_flutter_utilities/service/SyncRequest.dart';
import 'package:hotech_flutter_utilities/service/SyncResponse.dart';
import 'package:hotech_flutter_utilities/HFSharedPreferences.dart';
import 'package:http/http.dart';
import 'package:preferences/preference_service.dart';

class LoginService extends ServiceFactory {
  static LoginService instance;

  static LoginService getInstance() {
    return instance != null ? instance : LoginService();
  }

  HFSharedPreferences prefs;

  Future<LoginAuth> login(String username, String password) async {
    Response responseAuth = await callService(
      HttpMethod.POST,
      DBConst.FDN_AUTH + Consts.PATH_LOGIN,
      null,
      requestBody: {
        Consts.GRANT_TYPE: Consts.GRANT_TYPE_PASSWORD,
        Consts.CLIENT_ID: Consts.CLIENT_ID_OREST_CLIENT,
        Consts.CLIENT_SECRET: Consts.CLIENT_SECRET_TOP_SECRET,
        DBConst.FDN_USERNAME: username,
        DBConst.FDN_PASSWORD: password,
        DBConst.FDN_APP_NAME: Consts.ONAPP,
      },
    );

    if (responseAuth != null && ResponseType.isSuccessful(responseAuth.statusCode)) {
      return LoginUser.loginAuth = LoginAuth.fromJson(json.decode(responseAuth.body));
    }
    return null;
  }

  Future<SingleDataResult<Loginfo>> getLoginAuth(String username, String password) async {
    var responseAuth = await login(username, password);
    if (responseAuth != null) {
      (await HFSharedPreferences.getInstance()).saveLoginAuth(await login(username, password));
      return fetchLoginfo();
    }
    return null;
  }

  Future<SingleDataResult<Loginfo>> getRefreshTokenAuth({String refreshToken}) async {
    prefs = await HFSharedPreferences.getInstance();
    if (refreshToken == null) {
      refreshToken = await prefs.getRefreshToken();
    }
    Response responseAuth = await callService(
      HttpMethod.POST,
      DBConst.FDN_AUTH + Consts.PATH_LOGIN,
      null,
      requestBody: {
        Consts.GRANT_TYPE: Consts.GRANT_TYPE_REFRESH_TOKEN,
        Consts.CLIENT_ID: Consts.CLIENT_ID_OREST_CLIENT,
        Consts.CLIENT_SECRET: Consts.CLIENT_SECRET_TOP_SECRET,
        DBConst.FDN_REFRESH_TOKEN: refreshToken,
        DBConst.FDN_APP_NAME: Consts.ONAPP,
      },
    );

    if (responseAuth != null && ResponseType.isSuccessful(responseAuth.statusCode)) {
      prefs.saveLoginAuth(LoginUser.loginAuth = LoginAuth.fromJson(json.decode(responseAuth.body)));
      return fetchLoginfo();
    }
    return null;
  }

  Future<int> checkAuthCode() async {
    Response responseLoginfo = await callService(
      HttpMethod.GET,
      Consts.PARAM_INFO + Consts.PATH_LOGIN,
      await LoginHelper.getHeaders(),
    );
    return responseLoginfo.statusCode;
  }

  Future<SingleDataResult<Loginfo>> fetchLoginfo({Map<String, String> headers}) async {
    Response responseLoginfo = await callService(
      HttpMethod.GET,
      Consts.PARAM_INFO + Consts.PATH_LOGIN,
      headers ?? await LoginHelper.getHeaders(),
    );

    if (responseLoginfo != null && ResponseType.isSuccessful(responseLoginfo.statusCode)) {
      var resp = SingleDataResult<Loginfo>.fromJson(json.decode(responseLoginfo.body));
      (await HFSharedPreferences.getInstance()).saveLoginfo(LoginUser.logInfo = resp.data);
      return resp;
    } else {
      return null;
    }
  }

  @override
  String getTableName() {
    return null;
  }

  @override
  Future<List> getViewList({
    Map<String, dynamic> params,
    Map<String, dynamic> queries,
    Map<String, dynamic> headers,
  }) {
    return null;
  }

  @override
  Future<SingleDataResult<SyncResponse>> syncRequest(SyncRequest request) {
    return null;
  }

  Future<bool> logout() async {
    HFSharedPreferences prefs = await HFSharedPreferences.getInstance();

    Response response =
        await callService(HttpMethod.GET, DBConst.FDN_AUTH + Consts.PATH_LOGOUT, await LoginHelper.getHeaders());

    if (response != null && ResponseType.isSuccessful(response.statusCode)) {
      PrefService.setBool(Consts.PREF_KEY_REMEMBER_ME, false);
      prefs.invalidateTokens();
      return true;
    }
    return false;
  }
}
