import 'dart:convert';

import 'package:hotech_flutter_utilities/Constants.dart';
import 'package:hotech_flutter_utilities/model/reserv/ForecastDay.dart';
import 'package:hotech_flutter_utilities/model/ListDataResult.dart';
import 'package:hotech_flutter_utilities/service/LoginHelper.dart';
import 'package:hotech_flutter_utilities/service/ServiceFactory.dart';
import 'package:hotech_flutter_utilities/service/ServiceResponse.dart';
import 'package:http/http.dart';
import 'package:intl/intl.dart';

class ReservatService extends ServiceFactory<ForecastDay> {
  static ReservatService instance;

  static ReservatService getInstance() {
    return instance != null ? instance : ReservatService();
  }

  @override
  String getTableName() {
    return DBConst.TBL_RESERVAT;
  }

  Future<List<ForecastDay>> getForecastDay(DateTime startDate, DateTime endDate) async {
    Response response =
        await callService(HttpMethod.GET, Consts.PATH_FORECAST_DAY, await LoginHelper.getHeaders(), params: {
      DBConst.FDN_STARTDATE: DateFormat("dd-mm-yyyy").format(startDate),
      DBConst.FDN_ENDDATE: DateFormat("dd-mm-yyyy").format(endDate),
    });
    return response != null && ResponseType.isSuccessful(response.statusCode)
        ? ListDataResult<ForecastDay>.fromJson(json.decode(response.body)).data
        : null;
  }
}
